"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.transformDMMF = void 0;
const morphism_1 = require("morphism");
const ramda_1 = __importDefault(require("ramda"));
const helpers_1 = require("./helpers");
const properties_1 = require("./properties");
function transformDMMF(dmmf) {
    const enumIndex = ramda_1.default.indexBy(ramda_1.default.prop('name'), dmmf.datamodel.enums ?? []);
    const enumValuesToString = (e) => `ENUM(${e.values
        .map(ramda_1.default.prop('name'))
        .map((n) => `'${n}'`)
        .join(', ')})`;
    const scalarSchema = (0, morphism_1.createSchema)({
        isList: 'isList',
        hasDefaultValue: ramda_1.default.allPass([
            ramda_1.default.prop('hasDefaultValue'),
            ramda_1.default.compose(ramda_1.default.not, ramda_1.default.pathEq(['default', 'name'], 'dbgenerated')),
        ]),
        default: 'default',
        isId: 'isId',
        isUnique: 'isUnique',
        fieldName: 'name',
        type: (field) => field.kind === 'scalar'
            ? ramda_1.default.prop(field.type, properties_1.PrismaTypeToSequelizeType)
            : enumValuesToString(ramda_1.default.prop(field.type, enumIndex)),
        allowNull: ramda_1.default.anyPass([
            ramda_1.default.compose(ramda_1.default.not, ramda_1.default.prop('isRequired')),
            ramda_1.default.compose(ramda_1.default.pathEq(['default', 'name'], 'dbgenerated')),
        ]),
        isAutoincrement: ramda_1.default.allPass([ramda_1.default.prop('hasDefaultValue'), ramda_1.default.pathEq(['default', 'name'], 'autoincrement')]),
        isUuid: ramda_1.default.allPass([ramda_1.default.prop('hasDefaultValue'), ramda_1.default.pathEq(['default', 'name'], 'uuid')]),
        isNow: ramda_1.default.allPass([ramda_1.default.prop('hasDefaultValue'), ramda_1.default.pathEq(['default', 'name'], 'now')]),
    }, { undefinedValues: { strip: true } });
    const relationMorphism = (0, morphism_1.morphism)({
        as: 'name',
        name: 'type',
        targetKey: 'relationToFields[0]',
        foreignKey: 'relationFromFields[0]',
        relationName: 'relationName',
    });
    const modelMorphism = (0, morphism_1.morphism)({
        modelName: 'name',
        dbName: 'dbName',
        scalarFields: {
            path: 'fields',
            fn: (fields) => ramda_1.default.filter(ramda_1.default.allPass([
                ramda_1.default.propSatisfies((0, helpers_1.included)(['scalar', 'enum']), 'kind'),
                ramda_1.default.propSatisfies((0, helpers_1.notIncluded)(['createdAt', 'updatedAt', 'deletedAt']), 'name'),
            ]), fields).map((0, morphism_1.morphism)(scalarSchema)),
        },
        belongsToFields: {
            path: 'fields',
            fn: (fields) => ramda_1.default.filter(ramda_1.default.allPass([
                ramda_1.default.propEq('kind', 'object'),
                ramda_1.default.propSatisfies(ramda_1.default.not, 'isList'),
                ramda_1.default.propSatisfies(helpers_1.isNotEmpty, 'relationToFields'),
            ]), fields).map(relationMorphism),
        },
        hasOneFields: {
            path: 'fields',
            fn: (fields) => ramda_1.default.filter(ramda_1.default.allPass([
                ramda_1.default.propEq('kind', 'object'),
                ramda_1.default.propSatisfies(ramda_1.default.not, 'isList'),
                ramda_1.default.propSatisfies(ramda_1.default.isEmpty, 'relationToFields'),
            ]), fields).map(relationMorphism),
        },
        hasManyFields: {
            path: 'fields',
            fn: (fields) => ramda_1.default.filter(ramda_1.default.allPass([ramda_1.default.propEq('kind', 'object'), ramda_1.default.prop('isList')]), fields).map(relationMorphism),
        },
        hasCreatedAt: { path: 'fields', fn: ramda_1.default.compose(ramda_1.default.includes('createdAt'), ramda_1.default.map(ramda_1.default.prop('name'))) },
        hasUpdatedAt: { path: 'fields', fn: ramda_1.default.compose(ramda_1.default.includes('updatedAt'), ramda_1.default.map(ramda_1.default.prop('name'))) },
        hasDeletedAt: { path: 'fields', fn: ramda_1.default.compose(ramda_1.default.includes('deletedAt'), ramda_1.default.map(ramda_1.default.prop('name'))) },
    });
    const transformMorphism = (0, morphism_1.morphism)({
        models: { path: 'models', fn: modelMorphism },
    });
    const transformed = transformMorphism(dmmf.datamodel);
    // Link invertible hasMany/belongsTo relations
    for (const model of transformed.models) {
        for (const hasManyField of model.hasManyFields) {
            const { relationName } = hasManyField;
            const inversedRelation = transformed.models.reduce((found, model) => {
                const match = model.belongsToFields.find((belongsToField) => belongsToField.relationName === relationName);
                if (match) {
                    return match;
                }
                return found;
            }, null);
            if (inversedRelation) {
                hasManyField.foreignKey = inversedRelation.foreignKey;
                hasManyField.sourceKey = inversedRelation.targetKey;
            }
        }
    }
    return transformed;
}
exports.transformDMMF = transformDMMF;
